@@ -74,6 +74,7 @@ class DefaultDialect(interfaces.Dialect):
     supports_sequences = False
     sequences_optional = False
     preexecute_autoincrement_sequences = False
+    supports_identity_columns = False
     postfetch_lastrowid = True
     implicit_returning = False
     full_returning = False
@@ -808,6 +809,8 @@ class StrCompileDialect(DefaultDialect):
 
     supports_statement_cache = True
 
+    supports_identity_columns = True
+
     supports_sequences = True
     sequences_optional = True
     preexecute_autoincrement_sequences = False
@@ -1415,6 +1418,10 @@ class DefaultExecutionContext(interfaces.ExecutionContext):
                 strategy = _cursor._NO_CURSOR_DQL
 
             if self._is_future_result:
+                if self.root_connection.should_close_with_result:
+                    raise exc.InvalidRequestError(
+                        "can't use future_result=True with close_with_result"
+                    )
                 result = _cursor.CursorResult(
                     self, strategy, cursor_description
                 )
