@@ -270,6 +270,8 @@ class LambdaElement(elements.ClauseElement):
                     bind = bindparam_lookup[thing.key]
                     if thing.expanding:
                         bind.expanding = True
+                        bind.expand_op = thing.expand_op
+                        bind.type = thing.type
                     return bind
 
         if self._rec.is_sequence:
@@ -772,7 +774,16 @@ class AnalyzedCode(object):
         from the "track_on" parameter passed to a :class:`.LambdaElement`.
 
         """
-        if isinstance(elem, traversals.HasCacheKey):
+
+        if isinstance(elem, tuple):
+            # tuple must contain hascachekey elements
+            def get(closure, opts, anon_map, bindparams):
+                return tuple(
+                    tup_elem._gen_cache_key(anon_map, bindparams)
+                    for tup_elem in opts.track_on[idx]
+                )
+
+        elif isinstance(elem, traversals.HasCacheKey):
 
             def get(closure, opts, anon_map, bindparams):
                 return opts.track_on[idx]._gen_cache_key(anon_map, bindparams)
